@@ -29,35 +29,41 @@ Block Read/Write/Filter Functions
 from contextlib import contextmanager
 import logging
 
-from . import path
+from . import path, plugins
+
+# register builtin plugins and external plugins found in site-packages
+plugins.initialize()
 
 # initialize the paths
 try:
     path.find()
 except Exception as e:
-    logging.warn(str(e))
+    logging.warning(str(e))
 
 
 from . import ffmpegprocess
 
-from .utils.log import FFmpegError
+from .utils.error import FFmpegError
+from .utils.concat import FFConcat
 from .utils.filter import FilterGraph
+from . import devices, ffmpegprocess, caps, probe, audio, image, video, media
 from .transcode import transcode
-from . import caps, probe, audio, image, video, media
 from . import streams as _streams
 
 # fmt:off
-__all__ = ["ffmpeg_info", "get_path", "set_path", "is_ready", "transcode", "caps",
-    "probe", "audio", "image", "video", "media", "open", "ffmpegprocess",
-    "FFmpegError", "FilterGraph"]
+__all__ = ["ffmpeg_info", "get_path", "set_path", "is_ready", "ffmpeg", "ffprobe",
+    "transcode", "caps", "probe", "audio", "image", "video", "media", "devices",
+    "open", "ffmpegprocess", "FFmpegError", "FilterGraph", "FFConcat"]
 # fmt:on
 
-__version__ = "0.4.1"
+__version__ = "0.5.0"
 
-ffmpeg_info = ffmpegprocess.versions
+ffmpeg_info = path.versions
 set_path = path.find
 get_path = path.where
 is_ready = path.found
+ffmpeg = path.ffmpeg
+ffprobe = path.ffprobe
 
 
 @contextmanager
@@ -163,14 +169,14 @@ def open(
     =====  =====  =========  ===================================
     ncomp  dtype  pix_fmt    Description
     =====  =====  =========  ===================================
-      1     |u8   gray       grayscale
+      1     \|u8   gray       grayscale
       1     <u2   gray16le   16-bit grayscale
       1     <f4   grayf32le  floating-point grayscale
-      2     |u1   ya8        grayscale with alpha channel
+      2     \|u1   ya8        grayscale with alpha channel
       2     <u2   ya16le     16-bit grayscale with alpha channel
-      3     |u1   rgb24      RGB
+      3     \|u1   rgb24      RGB
       3     <u2   rgb48le    16-bit RGB
-      4     |u1   rgba       RGB with alpha transparency channel
+      4     \|u1   rgba       RGB with alpha transparency channel
       4     <u2   rgba64le   16-bit RGB with alpha channel
     =====  =====  =========  ===================================
 
@@ -181,7 +187,7 @@ def open(
     ======  ==========
     dtype   sample_fmt
     ======  ==========
-     |u1     u8
+     \|u1     u8
      <i2     s16
      <i4     s32
      <f4     flt
@@ -195,7 +201,9 @@ def open(
 
     """
 
+    is_fg = isinstance(url_fg, FilterGraph)
     if isinstance(url_fg, str):
+        is_fg = kwds.get("f_in", None) == "lavfi"
         url_fg = (url_fg,)
 
     audio = "a" in mode
@@ -226,12 +234,16 @@ def open(
 
     # auto-detect type
     if not (audio or video):
-        if read:
+        if is_fg:
+            raise ValueError(
+                "media type must be specified to read from an Input filtergraph"
+            )
+        elif read:
             for url in url_fg:
                 try:
                     info = probe.streams_basic(url, entries=("codec_type",))
                 except:
-                    raise ValueError(f"cannot be read url {url}")
+                    raise ValueError(f"cannot auto-detect media type of {url}")
                 for inf in info:
                     t = inf["codec_type"]
                     if t == "video" and not video:
@@ -248,7 +260,7 @@ def open(
                 audio = len(shape) < 2
             else:
                 # TODO identify based on file extension
-                raise ValueError("Unknown media type")
+                raise ValueError(f"cannot auto-detect media type")
             video = not audio
     elif read:
         # if audio or video is set multiple times, use avi reader
@@ -256,6 +268,8 @@ def open(
             video = audio and sum((1 for m in mode if m == "a")) > 1
         elif video and not audio:
             audio = video and sum((1 for m in mode if m == "v")) > 1
+    elif write and is_fg:
+        ValueError("Cannot write to a filtergraph.")
 
     try:
         StreamClass = {
