@@ -19,20 +19,19 @@ PIPE:    Special value that indicates a pipe should be created
 
 """
 
-import logging, re
-from os import path, devnull
+import logging
+from os import path
 from threading import Thread
 import subprocess as sp
-from subprocess import DEVNULL, PIPE
 from copy import deepcopy
 from tempfile import TemporaryDirectory
 
-from .ffmpeg import parse, compose
+from .utils.parser import parse, compose, FLAG
 from .threading import ProgressMonitorThread
 from .configure import move_global_options
-from .path import get_ffmpeg
+from .path import ffmpeg, DEVNULL, PIPE, devnull
 
-__all__ = ["versions", "run", "Popen", "PIPE", "DEVNULL", "devnull"]
+__all__ = ["versions", "run", "Popen", "FLAG", "PIPE", "DEVNULL", "devnull"]
 
 
 def exec(
@@ -80,17 +79,47 @@ def exec(
         ffmpeg_args = parse(ffmpeg_args)
 
     gopts = ffmpeg_args.get("global_options", None)
-    if hide_banner:
-        if gopts is None:
-            gopts = ffmpeg_args["global_options"] = {"hide_banner": None}
-        else:
-            gopts["hide_banner"] = None
+    if gopts is None:
+        gopts = ffmpeg_args["global_options"] = {}
 
+    # disable user-interaction by default
+    if "stdin" not in gopts:
+        gopts["nostdin"] = FLAG
+
+    # hide preamble by default
+    if hide_banner:
+        gopts["hide_banner"] = FLAG
+
+    # add URL to dump progress status
     if progress and progress.url:
-        if gopts is None:
-            ffmpeg_args["global_options"] = {"progress": progress.url}
+        gopts["progress"] = progress.url
+
+    # set y or n flags (overwrite)
+    if overwrite is not None:
+        if "y" in gopts or "n" in gopts:
+            raise ValueError(
+                "Cannot set both the overwrite argument and a y/n global flag."
+            )
+        elif overwrite:
+            gopts["y"] = FLAG
         else:
-            gopts["progress"] = progress.url
+            gopts["n"] = FLAG
+
+    # turn on hw decoder by default
+    def check_hwaccel(url, opts):
+        if opts is None:
+            opts = {"hwaccel": "auto"}
+        elif "hwaccel" not in opts:
+            opts["hwaccel"] = "auto"
+        return url, opts
+
+    if "inputs" in ffmpeg_args:
+        try:
+            ffmpeg_args["inputs"] = [
+                check_hwaccel(url, opts) for url, opts in ffmpeg_args["inputs"]
+            ]
+        except:
+            pass
 
     # configure stdin pipe (if needed)
     def isreadable(f):
@@ -145,63 +174,12 @@ def exec(
         PIPE if capture_log else None if capture_log is None else DEVNULL
     )
 
-    # set y or n flags (overwrite)
-    gopts = ffmpeg_args["global_options"]
-    if not (gopts and ("y" in gopts or "n" in gopts)):
-        if gopts is None:
-            gopts = ffmpeg_args["global_options"] = {}
-        if any((True for url, _ in ffmpeg_args.get("outputs", []) if url == devnull)):
-            gopts["y"] = None  # output to null, need to overwrite
-        elif overwrite is not None:
-            gopts["y" if overwrite else "n"] = None
-        else:
-            # set to no-overwrite by default to avoid hanging
-            gopts["n"] = None
-    elif overwrite is not None:
-        raise ValueError('Cannot set both the y/n global flag and the overwrite argument.')
-
-    args = compose(ffmpeg_args, command=get_ffmpeg())
-    logging.debug(args)
+    args = compose(ffmpeg_args)
 
     # run the FFmpeg
-    return sp_run(args, stdin=inpipe, stdout=outpipe, stderr=errpipe, **sp_kwargs)
-
-
-def versions():
-    """Get FFmpeg version and configuration information
-
-    :return: versions of ffmpeg and its av libraries as well as build configuration
-    :rtype: dict
-
-    ==================  ====  =========================================
-    key                 type  description
-    ==================  ====  =========================================
-    'version'           str   FFmpeg version
-    'configuration'     list  list of build configuration options
-    'library_versions'  dict  version numbers of dependent av libraries
-    ==================  ====  =========================================
-
-    """
-    s = exec(
-        ["-version"],
-        hide_banner=False,
-        stdout=PIPE,
-        universal_newlines=True,
-        encoding="utf-8",
-    ).stdout.splitlines()
-    v = dict(version=re.match(r"ffmpeg version (\S+)", s[0])[1])
-    i = 2 if s[1].startswith("built with") else 1
-    if s[i].startswith("configuration:"):
-        v["configuration"] = sorted([m[1] for m in re.finditer(r"\s--(\S+)", s[i])])
-        i += 1
-    lv = None
-    for l in s[i:]:
-        m = re.match(r"(\S+)\s+(.+?) /", l)
-        if m:
-            if lv is None:
-                lv = v["library_versions"] = {}
-            lv[m[1]] = m[2].replace(" ", "")
-    return v
+    return ffmpeg(
+        args, sp_run=sp_run, stdin=inpipe, stdout=outpipe, stderr=errpipe, **sp_kwargs
+    )
 
 
 def monitor_process(proc, on_exit=None):
@@ -257,14 +235,10 @@ class Popen(sp.Popen):
     :param \\**other_popen_args: other keyword arguments to :py:class:`subprocess.Popen`
     :type \\**other_popen_args: dict, optional
 
-    If :ref:`ffmpeg_args<adv_args>` calls for input or output to be piped (e.g., url="-") then :code:`Popen` creates
-    a pipe for each piped url. If input is piped, :code:`stdin` is default to :code:`ffmpegio.io.QueuedWriter`
-    class instance.  If output is piped, :code:`stdout` is default to :code:`ffmpegio.io.QueuedReader` class
-    instance. If :code:`capture_log=True`, then :code:`stderr` is default to :code:`ffmpegio.io.QueuedWriter`. See
-    :ref:`ffmpegio.io module<adv_io>` for how to use these custom stream classes.
-
-    Alternately, a file-stream object could be specified in the argument for each of :code:`stdin`, :code:`stdout`,
-    and :code:`stderr` to redirect pipes to existing file streams. If files aren't already open in Python,
+    If :ref:`ffmpeg_args<adv_args>` calls for input or output to be piped (e.g., url="-") then :code:`Popen` 
+    automatically sets `stdin=PIPE` or `stdout=PIPE`. Alternately, a file-stream object could be
+    specified in the argument for each of :code:`stdin`, :code:`stdout`, and :code:`stderr` 
+    to redirect pipes to existing file streams. If files aren't already open in Python,
     specify their urls in :ref:`ffmpeg_args<adv_args>` instead of using the pipes.
 
     """
@@ -290,8 +264,7 @@ class Popen(sp.Popen):
                 if k
                 in (
                     # fmt: off
-                    "bufsize", "executable",
-                    "close_fds", "shell", "niversal_newlines", "pass_fds",
+                    "executable", "close_fds", "shell", "niversal_newlines", "pass_fds",
                     "encoding", "errors", "text", "pipesize",
                     # fmt: on
                 )
@@ -464,7 +437,7 @@ def run(
         )
 
     # return stderr as str
-    if ret.stderr is not None:
+    if isinstance(ret.stderr, bytes):
         ret.stderr = ret.stderr.decode("utf-8")
 
     return ret
@@ -593,15 +566,16 @@ def run_two_pass(
         if stdin is not None:
             pos = stdin.tell()
 
-        run(pass1_args, **other_run_kwargs)
+        ret = run(pass1_args, **other_run_kwargs)
+        if not ret.returncode:
 
-        if stdin is not None:
-            stdin.seek(pos)
+            if stdin is not None:
+                stdin.seek(pos)
 
-        ret = run(ffmpeg_args, overwrite=overwrite, **other_run_kwargs)
+            ret = run(ffmpeg_args, overwrite=overwrite, **other_run_kwargs)
 
     # split log lines
-    if ret.stderr is not None:
+    if isinstance(ret.stderr, bytes):
         ret.stderr = ret.stderr.decode("utf-8")
 
     return ret
