@@ -1,62 +1,12 @@
-import logging, shlex, json, fractions, os, pickle, re
+import json, fractions, os, pickle, re
 from collections import OrderedDict
-import subprocess as sp
-from . import path
+from .path import ffprobe, PIPE
 
 # fmt:off
-__all__ = ['ffprobe', 'full_details', 'format_basic', 'streams_basic',
+__all__ = ['full_details', 'format_basic', 'streams_basic',
 'video_streams_basic', 'audio_streams_basic', 'query']
 # fmt:on
 
-form_shell_cmd = (
-    shlex.join
-    if hasattr(shlex, "join")
-    else lambda args: " ".join(shlex.quote(arg) for arg in args)
-)
-
-
-def ffprobe(
-    args,
-    *sp_arg,
-    stdout=sp.PIPE,
-    stderr=sp.PIPE,
-    universal_newlines=True,
-    encoding="utf8",
-    hide_banner=True,
-    **sp_kwargs,
-):
-    """run ffprobe command as a subprocess (blocking)
-
-    :param args: ffprobe argument options
-    :type args: seq or str
-    :param hide_banner: False to output ffmpeg banner, defaults to True
-    :type hide_banner: bool, optional
-    :return: ffprobe stdout output
-    :rtype: str
-    """
-    args = [
-        path.get_ffmpeg(probe=True),
-        *(["-hide_banner"] if hide_banner else []),
-        *(shlex.split(args) if isinstance(args, str) else args),
-    ]
-
-    logging.debug(form_shell_cmd(args))
-
-    ret = sp.run(
-        args,
-        *sp_arg,
-        stdout=stdout,
-        stderr=stderr,
-        universal_newlines=universal_newlines,
-        encoding=encoding,
-        **sp_kwargs,
-    )
-
-    if ret.returncode != 0:
-        raise Exception(f"execution failed\n   {form_shell_cmd(args)}\n\n{ret.stderr}")
-    return ret.stdout
-
-
 # stores all the local queries during the session
 # - key: path
 # - value: (mtime, blob)
@@ -82,6 +32,166 @@ def _items_to_numeric(d):
     return {k: try_conv(v) for k, v in d.items()}
 
 
+def _add_select_streams(args, stream_specifier):
+    if stream_specifier:
+        args.extend(["-select_streams", stream_specifier])
+    return args
+
+
+def _add_show_entries(args, entries):
+    arg = []
+    for key, val in entries.items():
+        if not isinstance(val, bool):
+            arg.append(f"{key}={','.join(val)}")
+        elif val:
+            arg.append(key)
+
+    args.append("-show_entries")
+    args.append(":".join(arg))
+    return args
+
+
+def _add_read_intervals(args, intervals):
+    """add -read_intervals option to ffprobe argumnets
+
+    :param args: argument list under construction
+    :type args: list[str]
+    :param intervals: interval specification
+    :type intervals: str, int, float, seq[str|float,str|int|float], dict, seq[dict]
+    :return: same as args input
+    :rtype: list[str]
+
+    ffprobe will seek to the interval starting point and will continue reading from that.
+    intervals argument can be specified in multiple ways to form the ffprobe argument:
+
+    1) ``str`` - pass through the argument as-is to ffprobe
+    2) ``int`` - read this numbers of packets to read from the beginning of the file
+    3) ``float`` - read packets over this duration in seconds from the beginning of the file
+    4) ``seq[str|float, str|int|float]`` - sets start and end points
+       - start: str = as-is, float=starting time in seconds
+       - end: str = as-is, int=offset in # of packets, float=offset in seconds
+    5) ``dict`` - specifies start and end points with the following keys:
+       - 'start'        - (str|float) start time
+       - 'start_offset' - (str|float) start time offset from the previous read. Ignored if 'start' is present.
+       - 'end'          - (str|float) end time
+       - 'end_offset'   - (str|float|int) end time offset from the start time. Ignored if 'end' is present.
+    6) - ``seq[dict]`` - specify multiple intervals
+
+    """
+
+    # INTERVAL  ::= [START|+START_OFFSET][%[END|+END_OFFSET]]
+    # INTERVALS ::= INTERVAL[,INTERVALS]
+    if intervals is None:
+        return args
+
+    def compose_time(t, is_end_offset):
+        if isinstance(t, str):
+            return str
+        elif isinstance(t, float):
+            return str(t)
+        elif is_end_offset and isinstance(t, int):
+            return f"#{t}"
+        else:
+            raise ValueError("unknown interval endpoint specification")
+
+    def compose_dict(intv):
+        s = ""
+        try:
+            s = compose_time(intv["start"], False)
+        except:
+            try:
+                s = "+" + compose_time(intv["start_offset"], False)
+            except:
+                pass
+        try:
+            s += "%" + compose_time(intv["end"], False)
+        except:
+            try:
+                s += "%+" + compose_time(intv["end_offset"], True)
+            except:
+                pass
+        return s
+
+    if not isinstance(intervals, str):
+        try:
+            if isinstance(intervals, dict):
+                intervals = compose_dict(intervals)
+            else:
+                try:
+                    # try to set duration
+                    intervals = f"%+{compose_time(intervals, True)}"
+                except:
+                    n = len(intervals)
+                    if n and isinstance(intervals[0], dict):
+                        # multiple intervals
+                        intervals = ",".join([compose_dict(intv) for intv in intervals])
+                    else:
+                        # (start, +end)
+                        assert len(intervals) == 2
+                        start = compose_time(intervals[0], False)
+                        end = compose_time(intervals[1], True)
+                        intervals = f"{start}%+{end}"
+        except:
+            raise ValueError("unknown interval specification")
+
+    args.extend(("-read_intervals", intervals))
+    return args
+
+
+def _exec(
+    url, entries, streams=None, intervals=None, count_frames=False, count_packets=False
+):
+    """execute ffprobe and return data as dict"""
+
+    sp_opts = {
+        "stdout": PIPE,
+        "stderr": PIPE,
+        "universal_newlines": True,
+        "encoding": "utf-8",
+    }
+
+    args = ["-hide_banner", "-of", "json"]
+
+    if streams is not None:
+        _add_select_streams(args, streams)
+
+    if intervals is not None:
+        _add_read_intervals(args, intervals)
+
+    if count_frames:
+        args.append("-count_frames")
+        # returns "nb_read_frames" item in each stream
+
+    if count_packets:
+        args.append("-count_packets")
+        # returns "nb_read_packets" item in each stream
+
+    _add_show_entries(args, entries)
+
+    pipe = not isinstance(url, str)
+    args.append("-" if pipe else url)
+
+    if pipe:
+        try:
+            assert url.seekable
+            pos0 = url.tell()
+            sp_opts["input"] = url
+        except:
+            raise ValueError("url must be str or seekable file-like object")
+
+    # run ffprobe
+    ret = ffprobe(args, **sp_opts)
+    if ret.returncode != 0:
+        raise Exception(f"ffprobe execution failed\n\n{ret.stderr}\n")
+
+    # decode output JSON string
+    results = json.loads(ret.stdout)
+
+    if pipe:
+        url.seek(pos0)
+    return results
+
+
 def _full_details(
     url,
     show_format=True,
@@ -109,44 +219,14 @@ def _full_details(
 
     """
 
-    args = ["-of", "json"]
-
-    if select_streams:
-        args.extend(["-select_streams", select_streams])
-
     modes = dict(
         format=show_format,
         stream=show_streams,
-        programs=show_programs,
-        chapters=show_chapters,
+        program=show_programs,
+        chapter=show_chapters,
     )
 
-    entries = []
-    for key, val in modes.items():
-        if not isinstance(val, bool):
-            entries.append(f"{key}={','.join(val)}")
-        elif val:
-            entries.append(key)
-        else:
-            entries.append(f"{key}=")
-
-    args.append("-show_entries")
-    args.append(":".join(entries))
-
-    pipe = not isinstance(url, str)
-    args.append("-" if pipe else url)
-
-    if pipe:
-        try:
-            assert not url.seekable
-            pos0 = url.tell()
-        except:
-            raise ValueError("url must be str or seekable io object")
-
-    results = json.loads(ffprobe(args))
-
-    if pipe:
-        url.seek(pos0)
+    results = _exec(url, modes, select_streams)
 
     if not modes["stream"]:
         modes["streams"] = modes["stream"]
@@ -157,25 +237,6 @@ def _full_details(
     return _items_to_numeric(results)
 
 
-def _queryall_if_path(url):
-
-    if re.match(r"[a-z][a-z0-9+.-]*://", url) is not None:
-        return None
-
-    mtime = os.stat(url).st_mtime
-
-    db_entry = _db.get(url, None)
-    if db_entry and db_entry[0] == mtime:
-        _db.move_to_end(url, True)  # refresh the entry position
-        return pickle.loads(db_entry[1])
-
-    results = _full_details(url, True, True, True, True)
-    _db[url] = (mtime, pickle.dumps(results))
-    if len(_db) > _db_maxsize:
-        _db.popitem(False)  # remove the oldest entry
-    return results
-
-
 def full_details(
     url,
     show_format=True,
@@ -203,54 +264,70 @@ def full_details(
 
     """
 
+    def _queryall_if_path():
+
+        assert isinstance(url, str)
+
+        sspec = None
+        if select_streams is not None:
+            try:
+                sspec = (None, int(select_streams))
+            except:
+                m = re.match("([avstd])(?::([0-9]+))?$", select_streams)
+                sspec = (
+                    {
+                        "a": "audio",
+                        "v": "video",
+                        "s": "subtitle",
+                        "t": "attachment",
+                        "d": "data",
+                    }[m[1]],
+                    m[2] and int(m[2]),
+                )
+                # raises exception if match not found
+
+        mtime = os.stat(url).st_mtime
+        db_entry = _db.get(url, None)
+        if db_entry and db_entry[0] == mtime:
+            _db.move_to_end(url, True)  # refresh the entry position
+            results = pickle.loads(db_entry[1])
+        else:
+            results = _full_details(url, True, True, True, True)
+            _db[url] = (mtime, pickle.dumps(results))
+            if len(_db) > _db_maxsize:
+                _db.popitem(False)  # remove the oldest entry
+
+        # drop unrequested items
+        for show, key in zip(
+            (show_format, show_streams, show_programs, show_chapters),
+            ("format", "streams", "programs", "chapters"),
+        ):
+            if not show:
+                del results[key]
+
+        # pick streams if specified
+        if sspec is not None:
+            t, i = sspec
+            streams = results["streams"]
+            if t is not None:
+                streams = [st for st in streams if st["codec_type"] == t]
+            if i is not None:
+                streams = streams[i : i + 1]
+            results["streams"] = streams
+
+        return results
+
     if not show_streams:
         select_streams = None
 
-    sspec = None
-
-    if isinstance(select_streams, int):
-        sspec = (None, select_streams)
-    elif select_streams is not None:
-        m = re.match("([avstd])(?::([0-9]+))?$", select_streams)
-        sspec = m and (
-            {
-                "a": "audio",
-                "v": "video",
-                "s": "subtitle",
-                "t": "attachment",
-                "d": "data",
-            }[m[1]],
-            m[2] and int(m[2]),
-        )
-
     # get full query if url is a path
-    results = (
-        _queryall_if_path(url) if select_streams is None or sspec is not None else None
-    )
-
-    if results is None:  # actual url, no store
+    try:
+        return _queryall_if_path()
+    except:
         return _full_details(
             url, show_format, show_streams, show_programs, show_chapters, select_streams
         )
 
-    for show, key in zip(
-        (show_format, show_streams, show_programs, show_chapters),
-        ("format", "streams", "programs", "chapters"),
-    ):
-        if not show:
-            del results[key]
-
-    if sspec is not None:
-        t, i = sspec
-        streams = results["streams"]
-        if t is not None:
-            streams = [st for st in streams if st["codec_type"] == t]
-        if i is not None:
-            streams = streams[i : i + 1]
-        results["streams"] = streams
-
-    return results
-
 
 def _resolve_entries(info_type, entries, default_entries, default_dep_entries={}):
 
@@ -570,18 +647,102 @@ def query(url, stream=None, fields=None, return_none=False):
         )
 
 
-# -show_data
-# -show_data_hash algorithm
-# -show_error
-# -show_packets
-# -show_frames
-# -show_log loglevel
-# -count_frames
-# -count_packets
-# -read_intervals read_intervals
-# -show_private_data, -private
-# -show_program_version
-# -show_library_versions
-# -show_versions
-# -show_pixel_formats
-# -bitexact
+def frames(url, entries=None, streams=None, intervals=None, accurate_time=False):
+    """get frame information
+
+    :param url: URL of the media file/stream
+    :type url: str or seekable file-like object
+    :param entries: names of frame attributes, defaults to None (get all attributes)
+    :type entries: str or seq[str], optional
+    :param stream: stream specifier of the stream to retrieve the data of, defaults to None to get all streams
+    :type stream: str or int, optional
+    :param intervals: time intervals to retrieve the data, see below for the details, defaults to None (get all)
+    :type intervals: str, int, float, seq[str|float,str|int|float], dict, seq[dict]
+    :param accurate_time: True to return all '*_time' attributes to be computed from associated timestamps and 
+                          stream timebase, defaults to False (= us accuracy)
+    :param accurate_time: bool, optional
+    :return: frame information. list of dictionary if entries is None or a sequence; list of the selected entry
+             if entries is str (i.e., a single entry)
+    :rtype: list[dict] or list[str|int|float]
+
+    ``intervals`` argument
+    ----------------------
+
+    intervals argument can be specified in multiple ways to form the ``-read_intervals`` ffprobe option:
+
+    1) ``str`` - pass through the argument as-is to ffprobe
+    2) ``int`` - read this numbers of packets to read from the beginning of the file
+    3) ``float`` - read packets over this duration in seconds from the beginning of the file
+    4) ``seq[str|float, str|int|float]`` - sets start and end points
+       - start: str = as-is, float=starting time in seconds
+       - end: str = as-is, int=offset in # of packets, float=offset in seconds
+    5) ``dict`` - specifies start and end points with the following keys:
+       - 'start'        - (str|float) start time
+       - 'start_offset' - (str|float) start time offset from the previous read. Ignored if 'start' is present.
+       - 'end'          - (str|float) end time
+       - 'end_offset'   - (str|float|int) end time offset from the start time. Ignored if 'end' is present.
+    6) - ``seq[dict]`` - specify multiple intervals
+
+    """
+
+    is_single = isinstance(entries, str)
+    if is_single:
+        entry = entries
+        entries = [entries]
+
+    if accurate_time:
+        pick_entries = entries is not None
+        has_time = not pick_entries
+        if pick_entries:
+            time_entries = []
+            other_entries = set(("stream_index",))
+            for e in entries:
+                if e.endswith("_time"):
+                    has_time = True
+                    time_entries.append(e)
+                    other_entries.add(e[:-5])
+                else:
+                    other_entries.add(e)
+            if has_time:
+                orig_entries = entries
+                entries = other_entries
+
+    entries = {"frame": (entries is None) or entries}
+    if accurate_time and has_time:
+        entries["stream"] = ["index", "time_base"]
+
+    res = _exec(
+        url,
+        entries,
+        streams=streams,
+        intervals=intervals,
+    )
+
+    out = [_items_to_numeric(d) for d in res["frames"]]
+
+    if len(out) == 0:
+        return out
+
+    if accurate_time and has_time:
+
+        time_bases = {
+            d["index"]: fractions.Fraction(d["time_base"])
+            for d in res["streams"]
+        }
+
+        if not pick_entries:
+            time_entries = [e for e in out[0].keys() if e.endswith("_time")]
+
+        ts_entries = [e[:-5] for e in time_entries]
+
+        for d in out:
+            tb = time_bases[d["stream_index"]]
+            for e, e_ts in zip(time_entries, ts_entries):
+                d[e] = d[e_ts] * tb
+                if pick_entries and e_ts not in orig_entries:
+                    del d[e_ts]
+
+    try:
+        return [d[entry] for d in out] if is_single else out
+    except:
+        raise ValueError(f"invalid frame attribute: {entry}")
