@@ -21,6 +21,7 @@ class SimpleReaderBase:
         show_log=None,
         progress=None,
         blocksize=None,
+        bufsize=-1,
         **options,
     ) -> None:
 
@@ -53,7 +54,11 @@ class SimpleReaderBase:
 
         # start FFmpeg
         self._proc = ffmpegprocess.Popen(
-            ffmpeg_args, stdin=stdin, progress=progress, capture_log=True
+            ffmpeg_args,
+            stdin=stdin,
+            progress=progress,
+            capture_log=True,
+            bufsize=bufsize,
         )
 
         # set the log source and start the logger
@@ -195,7 +200,7 @@ class SimpleVideoReader(SimpleReaderBase):
     multi_read = False
     multi_write = False
 
-    def __init__(self, url, show_log=None, progress=None, blocksize=None, **options):
+    def __init__(self, url, show_log=None, progress=None, blocksize=1, **options):
         hook = plugins.get_hook()
         super().__init__(
             hook.bytes_to_video,
@@ -215,18 +220,25 @@ class SimpleVideoReader(SimpleReaderBase):
         has_fg = configure.has_filtergraph(ffmpeg_args, "video")
 
         pix_fmt = outopts.get("pix_fmt", None)
-        if pix_fmt is None or (
-            not has_fg
+        pix_fmt_in = s_in = r_in = None
+        if (
+            pix_fmt is None
+            and not has_fg
             and inurl not in ("-", "pipe:", "pipe:0")
             and not inopts.get("pix_fmt", None)
         ):
-            # must assign output rgb/grayscale pixel format
-            info = probe.video_streams_basic(inurl, 0)[0]
-            pix_fmt_in = info["pix_fmt"]
-            s_in = (info["width"], info["height"])
-            r_in = info["frame_rate"]
-        else:
-            pix_fmt_in = s_in = r_in = None
+            try:
+                # must assign output rgb/grayscale pixel format
+                info = probe.video_streams_basic(inurl, 0)[0]
+                pix_fmt_in = info["pix_fmt"]
+                s_in = (info["width"], info["height"])
+                r_in = info["frame_rate"]
+            except:
+                pix_fmt_in = 'rgb24'
+            
+
+        if pix_fmt_in is None and pix_fmt is None:
+            raise ValueError("pix_fmt must be specified.")
 
         (
             self.dtype,
@@ -269,7 +281,7 @@ class SimpleAudioReader(SimpleReaderBase):
         # finalize FFmpeg arguments and output array
 
         inurl, inopts = ffmpeg_args.get("inputs", [])[0]
-        has_fg = configure.has_filtergraph(ffmpeg_args, "video")
+        has_fg = configure.has_filtergraph(ffmpeg_args, "audio")
 
         sample_fmt_in = inopts.get("sample_fmt", None)
         ac_in = ar_in = None
@@ -281,7 +293,7 @@ class SimpleAudioReader(SimpleReaderBase):
                 ac_in = info.get("channels", None)
                 ar_in = info.get("sample_rate", None)
             except:
-                pass
+                sample_fmt_in = 's16'
 
         (
             self.dtype,
