@@ -11,7 +11,8 @@ from queue import Empty, Full, Queue
 from math import ceil
 
 from .utils.avi import AviReader
-from .utils.log import extract_output_stream as _extract_output_stream, FFmpegError
+from .utils.log import extract_output_stream as _extract_output_stream
+from .utils.error import FFmpegError
 
 # fmt:off
 __all__ = ['AviReader', 'FFmpegError', 'ThreadNotActive', 'ProgressMonitorThread',
